import { IEncryptor, PublicKeyRaws } from './encryptor';
import { TxState } from './tx-state';
import { GameEvent } from './events';
export type ConnectionState = 'disconnected' | 'connected' | 'reconnected' | 'closed';
type Method = 'attach_game' | 'submit_event' | 'exit_game' | 'subscribe_event' | 'submit_message' | 'ping';
interface IAttachGameParams {
    signer: string;
    key: PublicKeyRaws;
}
interface ISubscribeEventParams {
    settleVersion: bigint;
}
interface ISubmitEventParams {
    event: GameEvent;
}
interface ISubmitMessageParams {
    content: string;
}
export declare class AttachGameParams {
    signer: string;
    key: PublicKeyRaws;
    constructor(fields: IAttachGameParams);
}
export declare class ExitGameParams {
    keepConnection?: boolean;
}
export declare class SubscribeEventParams {
    settleVersion: bigint;
    constructor(fields: ISubscribeEventParams);
}
export declare class SubmitEventParams {
    event: GameEvent;
    constructor(fields: ISubmitEventParams);
}
export declare class SubmitMessageParams {
    content: string;
    constructor(fields: ISubmitMessageParams);
}
export declare class Message {
    sender: string;
    content: string;
    constructor(fields: any);
}
export declare abstract class BroadcastFrame {
}
export declare class BroadcastFrameEvent extends BroadcastFrame {
    gameAddr: string;
    event: GameEvent;
    timestamp: bigint;
    constructor(fields: any);
}
export declare class BroadcastFrameMessage extends BroadcastFrame {
    gameAddr: string;
    message: Message;
    constructor(fields: any);
}
export declare class BroadcastFrameTxState extends BroadcastFrame {
    txState: TxState;
    constructor(fields: any);
}
export interface IConnection {
    attachGame(params: AttachGameParams): Promise<void>;
    submitEvent(params: SubmitEventParams): Promise<ConnectionState | undefined>;
    submitMessage(params: SubmitMessageParams): Promise<ConnectionState | undefined>;
    exitGame(params: ExitGameParams): Promise<void>;
    connect(params: SubscribeEventParams): Promise<void>;
    subscribeEvents(): AsyncGenerator<BroadcastFrame | ConnectionState | undefined>;
}
export declare class Connection implements IConnection {
    gameAddr: string;
    playerAddr: string;
    endpoint: string;
    encryptor: IEncryptor;
    socket?: WebSocket;
    closed: boolean;
    streamResolve?: ((value: BroadcastFrame | ConnectionState | undefined) => void);
    streamMessageQueue: BroadcastFrame[];
    streamMessagePromise?: Promise<BroadcastFrame | ConnectionState | undefined>;
    lastPong: number;
    checkTimer?: any;
    isFirstOpen: boolean;
    constructor(gameAddr: string, playerAddr: string, endpoint: string, encryptor: IEncryptor);
    onDisconnected(): void;
    clearCheckTimer(): void;
    connect(params: SubscribeEventParams): Promise<void>;
    attachGame(params: AttachGameParams): Promise<void>;
    submitEvent(params: SubmitEventParams): Promise<ConnectionState | undefined>;
    submitMessage(params: SubmitMessageParams): Promise<ConnectionState | undefined>;
    exitGame(params: ExitGameParams): Promise<void>;
    subscribeEvents(): AsyncGenerator<BroadcastFrame | ConnectionState | undefined>;
    parseEventMessage(raw: string): BroadcastFrame | ConnectionState | undefined;
    static initialize(gameAddr: string, playerAddr: string, endpoint: string, encryptor: IEncryptor): Connection;
    makeReq<P>(gameAddr: string, method: Method, params: P): Promise<string>;
    makeReqNoSig<P>(gameAddr: string, method: Method, params: P): string;
    requestWs(req: string): Promise<void>;
    requestXhr<P>(req: string): Promise<P>;
    waitSocketReady(): Promise<unknown>;
}
export {};
