"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTtlCache = exports.setTtlCache = void 0;
/**
 * Set a cache with a `key`, and expire after `ttl` miliseconds.
 */
function setTtlCache(storage, key, value, ttl) {
    const data = {
        expire: new Date().getTime() + ttl,
        value
    };
    storage.setItem(key, JSON.stringify(data));
}
exports.setTtlCache = setTtlCache;
/**
 * Get a TTL cache value by `key`.
 */
function getTtlCache(storage, key) {
    const s = storage.getItem(key);
    if (!!s) {
        const data = JSON.parse(s);
        if (data.expire < new Date().getTime()) {
            return undefined;
        }
        return data.value;
    }
    return undefined;
}
exports.getTtlCache = getTtlCache;
//# sourceMappingURL=storage.js.map