var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _AppClient_gameAddr, _AppClient_handler, _AppClient_wallet, _AppClient_client, _AppClient_transport, _AppClient_connection, _AppClient_gameContext, _AppClient_onEvent, _AppClient_onMessage, _AppClient_onTxState, _AppClient_onConnectionState, _AppClient_encryptor, _AppClient_profileCaches, _AppClient_info, _AppClient_decryptionCache;
import { BroadcastFrameEvent, BroadcastFrameMessage, BroadcastFrameTxState, Connection, SubmitEventParams, SubmitMessageParams, SubscribeEventParams, } from './connection';
import { GameContext } from './game-context';
import { GameContextSnapshot } from './game-context-snapshot';
import { Handler, InitAccount } from './handler';
import { Encryptor } from './encryptor';
import { SdkError } from './error';
import { EntryTypeCash } from './accounts';
import { Client } from './client';
import { Custom } from './events';
import { ProfileCache } from './profile-cache';
import { getTtlCache, setTtlCache } from './storage';
import { DecryptionCache } from './decryption-cache';
const BUNDLE_CACHE_TTL = 3600 * 365;
export class AppClient {
    constructor(gameAddr, handler, wallet, client, transport, connection, gameContext, onEvent, onMessage, onTxState, onConnectionState, encryptor, info, decryptionCache) {
        _AppClient_gameAddr.set(this, void 0);
        _AppClient_handler.set(this, void 0);
        _AppClient_wallet.set(this, void 0);
        _AppClient_client.set(this, void 0);
        _AppClient_transport.set(this, void 0);
        _AppClient_connection.set(this, void 0);
        _AppClient_gameContext.set(this, void 0);
        _AppClient_onEvent.set(this, void 0);
        _AppClient_onMessage.set(this, void 0);
        _AppClient_onTxState.set(this, void 0);
        _AppClient_onConnectionState.set(this, void 0);
        _AppClient_encryptor.set(this, void 0);
        _AppClient_profileCaches.set(this, void 0);
        _AppClient_info.set(this, void 0);
        _AppClient_decryptionCache.set(this, void 0);
        __classPrivateFieldSet(this, _AppClient_gameAddr, gameAddr, "f");
        __classPrivateFieldSet(this, _AppClient_handler, handler, "f");
        __classPrivateFieldSet(this, _AppClient_wallet, wallet, "f");
        __classPrivateFieldSet(this, _AppClient_client, client, "f");
        __classPrivateFieldSet(this, _AppClient_transport, transport, "f");
        __classPrivateFieldSet(this, _AppClient_connection, connection, "f");
        __classPrivateFieldSet(this, _AppClient_gameContext, gameContext, "f");
        __classPrivateFieldSet(this, _AppClient_onEvent, onEvent, "f");
        __classPrivateFieldSet(this, _AppClient_onMessage, onMessage, "f");
        __classPrivateFieldSet(this, _AppClient_onTxState, onTxState, "f");
        __classPrivateFieldSet(this, _AppClient_onConnectionState, onConnectionState, "f");
        __classPrivateFieldSet(this, _AppClient_encryptor, encryptor, "f");
        __classPrivateFieldSet(this, _AppClient_profileCaches, new ProfileCache(transport), "f");
        __classPrivateFieldSet(this, _AppClient_info, info, "f");
        __classPrivateFieldSet(this, _AppClient_decryptionCache, decryptionCache, "f");
    }
    static async initialize(opts) {
        const { transport, wallet, gameAddr, onEvent, onMessage, onTxState, onConnectionState, storage } = opts;
        console.group('AppClient initialization');
        try {
            const playerAddr = wallet.walletAddr;
            const encryptor = await Encryptor.create(playerAddr, storage);
            const gameAccount = await transport.getGameAccount(gameAddr);
            if (gameAccount === undefined) {
                throw SdkError.gameAccountNotFound(gameAddr);
            }
            console.log('Game account:', gameAccount);
            // Fetch game bundle
            // The bundle can be considered as immutable, so we use cache whenever possible
            const bundleCacheKey = `BUNDLE__${transport.chain}_${gameAccount.bundleAddr}`;
            let gameBundle;
            if (storage !== undefined) {
                gameBundle = getTtlCache(storage, bundleCacheKey);
                console.log('Use game bundle from cache:', gameBundle);
                if (gameBundle !== undefined) {
                    Object.assign(gameBundle, { data: Uint8Array.of() });
                }
            }
            if (gameBundle === undefined) {
                gameBundle = await transport.getGameBundle(gameAccount.bundleAddr);
                console.log('Game bundle:', gameBundle);
                if (gameBundle !== undefined && storage !== undefined && gameBundle.data.length === 0) {
                    setTtlCache(storage, bundleCacheKey, gameBundle, BUNDLE_CACHE_TTL);
                }
            }
            if (gameBundle === undefined) {
                throw SdkError.gameBundleNotFound(gameAccount.bundleAddr);
            }
            const transactorAddr = gameAccount.transactorAddr;
            if (transactorAddr === undefined) {
                throw SdkError.gameNotServed(gameAddr);
            }
            console.log('Transactor address:', transactorAddr);
            const transactorAccount = await transport.getServerAccount(transactorAddr);
            if (transactorAccount === undefined) {
                throw SdkError.transactorAccountNotFound(transactorAddr);
            }
            const decryptionCache = new DecryptionCache();
            console.log('Transactor endpoint:', transactorAccount.endpoint);
            const connection = Connection.initialize(gameAddr, playerAddr, transactorAccount.endpoint, encryptor);
            const client = new Client(playerAddr, gameAddr, encryptor, connection);
            const handler = await Handler.initialize(gameBundle, encryptor, client, decryptionCache);
            const gameContext = new GameContext(gameAccount);
            gameContext.applyCheckpoint(gameContext.checkpointAccessVersion, gameContext.settleVersion);
            const token = await transport.getToken(gameAccount.tokenAddr);
            if (token === undefined) {
                throw SdkError.tokenNotFound(gameAccount.tokenAddr);
            }
            const info = {
                title: gameAccount.title,
                entryType: gameAccount.entryType,
                maxPlayers: gameAccount.maxPlayers,
                tokenAddr: gameAccount.tokenAddr,
                bundleAddr: gameAccount.bundleAddr,
                data: gameAccount.data,
                dataLen: gameAccount.dataLen,
                token,
            };
            if (gameAccount.entryType instanceof EntryTypeCash) {
                info.minDeposit = gameAccount.entryType.minDeposit;
                info.maxDeposit = gameAccount.entryType.maxDeposit;
            }
            return new AppClient(gameAddr, handler, wallet, client, transport, connection, gameContext, onEvent, onMessage, onTxState, onConnectionState, encryptor, info, decryptionCache);
        }
        finally {
            console.groupEnd();
        }
    }
    get playerAddr() {
        return __classPrivateFieldGet(this, _AppClient_wallet, "f").walletAddr;
    }
    get gameAddr() {
        return __classPrivateFieldGet(this, _AppClient_gameAddr, "f");
    }
    get gameContext() {
        return __classPrivateFieldGet(this, _AppClient_gameContext, "f");
    }
    /**
     * Get player profile by its wallet address.
     */
    async getProfile(addr) {
        return await __classPrivateFieldGet(this, _AppClient_transport, "f").getPlayerProfile(addr);
    }
    async invokeEventCallback(event) {
        const snapshot = new GameContextSnapshot(__classPrivateFieldGet(this, _AppClient_gameContext, "f"));
        await __classPrivateFieldGet(this, _AppClient_profileCaches, "f").injectProfiles(snapshot);
        const state = __classPrivateFieldGet(this, _AppClient_gameContext, "f").handlerState;
        __classPrivateFieldGet(this, _AppClient_onEvent, "f").call(this, snapshot, state, event);
    }
    async __initializeState(gameAccount) {
        const initAccount = InitAccount.createFromGameAccount(gameAccount, this.gameContext.accessVersion, this.gameContext.settleVersion);
        console.log('Initialize state with', initAccount);
        await __classPrivateFieldGet(this, _AppClient_handler, "f").initState(__classPrivateFieldGet(this, _AppClient_gameContext, "f"), initAccount);
        await this.invokeEventCallback(undefined);
    }
    async __getGameAccount() {
        while (true) {
            try {
                const gameAccount = await __classPrivateFieldGet(this, _AppClient_transport, "f").getGameAccount(this.gameAddr);
                if (gameAccount === undefined)
                    continue;
                console.log('Game account', gameAccount);
                return gameAccount;
            }
            catch (e) {
                console.warn(e, 'Failed to fetch game account, will retry in 3s');
                await new Promise(r => setTimeout(r, 3000));
                continue;
            }
        }
    }
    /**
     * Connect to the transactor and retrieve the event stream.
     */
    async attachGame() {
        await __classPrivateFieldGet(this, _AppClient_client, "f").attachGame();
        const sub = __classPrivateFieldGet(this, _AppClient_connection, "f").subscribeEvents();
        const gameAccount = await this.__getGameAccount();
        __classPrivateFieldSet(this, _AppClient_gameContext, new GameContext(gameAccount), "f");
        __classPrivateFieldGet(this, _AppClient_gameContext, "f").applyCheckpoint(gameAccount.checkpointAccessVersion, __classPrivateFieldGet(this, _AppClient_gameContext, "f").settleVersion);
        await __classPrivateFieldGet(this, _AppClient_connection, "f").connect(new SubscribeEventParams({ settleVersion: __classPrivateFieldGet(this, _AppClient_gameContext, "f").settleVersion }));
        await this.__initializeState(gameAccount);
        for await (const frame of sub) {
            if (frame instanceof BroadcastFrameMessage) {
                console.group('Receive message');
                try {
                    if (__classPrivateFieldGet(this, _AppClient_onMessage, "f") !== undefined) {
                        const { message } = frame;
                        __classPrivateFieldGet(this, _AppClient_onMessage, "f").call(this, message);
                    }
                }
                finally {
                    console.groupEnd();
                }
            }
            else if (frame instanceof BroadcastFrameTxState) {
                console.group('Receive tx state');
                try {
                    if (__classPrivateFieldGet(this, _AppClient_onTxState, "f") !== undefined) {
                        const { txState } = frame;
                        __classPrivateFieldGet(this, _AppClient_onTxState, "f").call(this, txState);
                    }
                }
                finally {
                    console.groupEnd();
                }
            }
            else if (frame instanceof BroadcastFrameEvent) {
                const { event, timestamp } = frame;
                console.group('Handle event: ' + event.kind() + ' at timestamp: ' + new Date(Number(timestamp)).toLocaleString());
                console.log('Event: ', event);
                try {
                    __classPrivateFieldGet(this, _AppClient_gameContext, "f").prepareForNextEvent(timestamp);
                    try {
                        let context = new GameContext(__classPrivateFieldGet(this, _AppClient_gameContext, "f"));
                        await __classPrivateFieldGet(this, _AppClient_handler, "f").handleEvent(context, event);
                        __classPrivateFieldSet(this, _AppClient_gameContext, context, "f");
                    }
                    catch (err) {
                        console.error(err);
                    }
                    await this.invokeEventCallback(event);
                }
                catch (e) {
                    console.log("Game context in error:", __classPrivateFieldGet(this, _AppClient_gameContext, "f"));
                    throw e;
                }
                finally {
                    console.groupEnd();
                }
            }
            else if (frame === 'disconnected') {
                if (__classPrivateFieldGet(this, _AppClient_onConnectionState, "f") !== undefined) {
                    __classPrivateFieldGet(this, _AppClient_onConnectionState, "f").call(this, 'disconnected');
                }
                console.group('Disconnected, try reset state and context');
                try {
                    const gameAccount = await this.__getGameAccount();
                    __classPrivateFieldSet(this, _AppClient_gameContext, new GameContext(gameAccount), "f");
                    __classPrivateFieldGet(this, _AppClient_gameContext, "f").applyCheckpoint(gameAccount.checkpointAccessVersion, __classPrivateFieldGet(this, _AppClient_gameContext, "f").settleVersion);
                    await __classPrivateFieldGet(this, _AppClient_connection, "f").connect(new SubscribeEventParams({ settleVersion: __classPrivateFieldGet(this, _AppClient_gameContext, "f").settleVersion }));
                    await this.__initializeState(gameAccount);
                }
                finally {
                    console.groupEnd();
                }
            }
            else if (frame === 'connected') {
                if (__classPrivateFieldGet(this, _AppClient_onConnectionState, "f") !== undefined) {
                    __classPrivateFieldGet(this, _AppClient_onConnectionState, "f").call(this, 'connected');
                }
            }
            else if (frame === 'closed') {
                if (__classPrivateFieldGet(this, _AppClient_onConnectionState, "f") !== undefined) {
                    __classPrivateFieldGet(this, _AppClient_onConnectionState, "f").call(this, 'closed');
                }
            }
            else if (frame === 'reconnected') {
                if (__classPrivateFieldGet(this, _AppClient_onConnectionState, "f") !== undefined) {
                    __classPrivateFieldGet(this, _AppClient_onConnectionState, "f").call(this, 'reconnected');
                }
            }
            else {
                console.log('Subscribe stream ended');
                break;
            }
        }
    }
    /**
     * Join game.
     */
    async join(params) {
        const gameAccount = await __classPrivateFieldGet(this, _AppClient_transport, "f").getGameAccount(this.gameAddr);
        if (gameAccount === undefined) {
            throw new Error('Game account not found');
        }
        const playersCount = gameAccount.players.length;
        if (gameAccount.maxPlayers <= playersCount) {
            throw new Error('Game is full');
        }
        let position = params.position;
        if (position === undefined) {
            for (let i = 0; i < gameAccount.maxPlayers; i++) {
                if (gameAccount.players.find(p => p.position === i) === undefined) {
                    position = i;
                    break;
                }
            }
        }
        if (position === undefined) {
            throw new Error('Game is full');
        }
        const publicKey = await __classPrivateFieldGet(this, _AppClient_encryptor, "f").exportPublicKey();
        return await __classPrivateFieldGet(this, _AppClient_transport, "f").join(__classPrivateFieldGet(this, _AppClient_wallet, "f"), {
            gameAddr: this.gameAddr,
            amount: params.amount,
            accessVersion: gameAccount.accessVersion,
            position,
            verifyKey: publicKey.ec,
        });
    }
    async submitEvent(arg) {
        let raw = arg instanceof Uint8Array ? arg : arg.serialize();
        const event = new Custom({ sender: this.playerAddr, raw });
        const connState = await __classPrivateFieldGet(this, _AppClient_connection, "f").submitEvent(new SubmitEventParams({
            event,
        }));
        if (connState !== undefined && __classPrivateFieldGet(this, _AppClient_onConnectionState, "f") !== undefined) {
            __classPrivateFieldGet(this, _AppClient_onConnectionState, "f").call(this, connState);
        }
    }
    /**
     * Submit a message, contains arbitrary content.
     */
    async submitMessage(message) {
        const connState = await __classPrivateFieldGet(this, _AppClient_connection, "f").submitMessage(new SubmitMessageParams({
            content: message,
        }));
        if (connState !== undefined && __classPrivateFieldGet(this, _AppClient_onConnectionState, "f") !== undefined) {
            __classPrivateFieldGet(this, _AppClient_onConnectionState, "f").call(this, connState);
        }
    }
    /**
     * Get hidden knowledge by random id. The result contains both
     * public and private information.  For performance reason, it's
     * better to cache the result somewhere instead of calling this
     * function frequently.
     */
    async getRevealed(randomId) {
        return __classPrivateFieldGet(this, _AppClient_decryptionCache, "f").get(randomId) || new Map();
    }
    /**
     * Close current event subscription.
     */
    async close() { }
    async exit(keepConnection = false) {
        await __classPrivateFieldGet(this, _AppClient_connection, "f").exitGame({ keepConnection });
    }
    get info() {
        return __classPrivateFieldGet(this, _AppClient_info, "f");
    }
}
_AppClient_gameAddr = new WeakMap(), _AppClient_handler = new WeakMap(), _AppClient_wallet = new WeakMap(), _AppClient_client = new WeakMap(), _AppClient_transport = new WeakMap(), _AppClient_connection = new WeakMap(), _AppClient_gameContext = new WeakMap(), _AppClient_onEvent = new WeakMap(), _AppClient_onMessage = new WeakMap(), _AppClient_onTxState = new WeakMap(), _AppClient_onConnectionState = new WeakMap(), _AppClient_encryptor = new WeakMap(), _AppClient_profileCaches = new WeakMap(), _AppClient_info = new WeakMap(), _AppClient_decryptionCache = new WeakMap();
//# sourceMappingURL=app-client.js.map