var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { PublicKeyRaws } from './encryptor';
import { TxState } from './tx-state';
import { GameEvent } from './events';
import { deserialize, enums, field, serialize, struct, variant } from '@race-foundation/borsh';
import { arrayBufferToBase64, base64ToUint8Array } from './utils';
export class AttachGameParams {
    constructor(fields) {
        this.key = fields.key;
        this.signer = fields.signer;
    }
}
__decorate([
    field('string')
], AttachGameParams.prototype, "signer", void 0);
__decorate([
    field(struct(PublicKeyRaws))
], AttachGameParams.prototype, "key", void 0);
export class ExitGameParams {
}
export class SubscribeEventParams {
    constructor(fields) {
        this.settleVersion = fields.settleVersion;
    }
}
__decorate([
    field('u64')
], SubscribeEventParams.prototype, "settleVersion", void 0);
export class SubmitEventParams {
    constructor(fields) {
        this.event = fields.event;
    }
}
__decorate([
    field(enums(GameEvent))
], SubmitEventParams.prototype, "event", void 0);
export class SubmitMessageParams {
    constructor(fields) {
        this.content = fields.content;
    }
}
__decorate([
    field('string')
], SubmitMessageParams.prototype, "content", void 0);
export class Message {
    constructor(fields) {
        Object.assign(this, fields);
    }
}
__decorate([
    field('string')
], Message.prototype, "sender", void 0);
__decorate([
    field('string')
], Message.prototype, "content", void 0);
export class BroadcastFrame {
}
let BroadcastFrameEvent = class BroadcastFrameEvent extends BroadcastFrame {
    constructor(fields) {
        super();
        Object.assign(this, fields);
    }
};
__decorate([
    field('string')
], BroadcastFrameEvent.prototype, "gameAddr", void 0);
__decorate([
    field(enums(GameEvent))
], BroadcastFrameEvent.prototype, "event", void 0);
__decorate([
    field('u64')
], BroadcastFrameEvent.prototype, "timestamp", void 0);
BroadcastFrameEvent = __decorate([
    variant(0)
], BroadcastFrameEvent);
export { BroadcastFrameEvent };
let BroadcastFrameMessage = class BroadcastFrameMessage extends BroadcastFrame {
    constructor(fields) {
        super();
        Object.assign(this, fields);
    }
};
__decorate([
    field('string')
], BroadcastFrameMessage.prototype, "gameAddr", void 0);
__decorate([
    field(struct(Message))
], BroadcastFrameMessage.prototype, "message", void 0);
BroadcastFrameMessage = __decorate([
    variant(1)
], BroadcastFrameMessage);
export { BroadcastFrameMessage };
let BroadcastFrameTxState = class BroadcastFrameTxState extends BroadcastFrame {
    constructor(fields) {
        super();
        Object.assign(this, fields);
    }
};
__decorate([
    field(enums(TxState))
], BroadcastFrameTxState.prototype, "txState", void 0);
BroadcastFrameTxState = __decorate([
    variant(2)
], BroadcastFrameTxState);
export { BroadcastFrameTxState };
export class Connection {
    constructor(gameAddr, playerAddr, endpoint, encryptor) {
        this.gameAddr = gameAddr;
        this.playerAddr = playerAddr;
        this.endpoint = endpoint;
        this.encryptor = encryptor;
        this.socket = undefined;
        this.closed = false;
        this.streamResolve = undefined;
        this.streamMessageQueue = [];
        this.streamMessagePromise = undefined;
        this.lastPong = new Date().getTime();
        this.isFirstOpen = true;
    }
    onDisconnected() {
        console.warn('Clean up the connection with transactor');
        this.clearCheckTimer();
        if (this.socket === undefined) {
            return;
        }
        else {
            this.socket.close();
            this.socket = undefined;
        }
        if (this.streamMessageQueue.find(x => x === 'disconnected') === undefined) {
            if (this.streamResolve !== undefined) {
                let r = this.streamResolve;
                this.streamResolve = undefined;
                r('disconnected');
            }
            else {
                this.streamMessageQueue.push('disconnected');
            }
        }
    }
    clearCheckTimer() {
        if (this.checkTimer !== undefined) {
            clearInterval(this.checkTimer);
            this.checkTimer = undefined;
        }
    }
    async connect(params) {
        console.log('Establishing server connection, settle version:', params.settleVersion);
        this.socket = new WebSocket(this.endpoint);
        this.clearCheckTimer();
        this.socket.onmessage = msg => {
            const frame = this.parseEventMessage(msg.data);
            if (frame !== undefined) {
                if (this.streamResolve !== undefined) {
                    let r = this.streamResolve;
                    this.streamResolve = undefined;
                    r(frame);
                }
                else {
                    this.streamMessageQueue.push(frame);
                }
            }
        };
        this.socket.onopen = () => {
            console.log('Websocket connected');
            let frame;
            if (this.isFirstOpen) {
                frame = 'connected';
                this.isFirstOpen = false;
            }
            else {
                frame = 'reconnected';
            }
            // Start times for alive checking
            this.lastPong = new Date().getTime();
            this.checkTimer = setInterval(() => {
                const t = new Date().getTime();
                if (this.lastPong + 6000 < t) {
                    console.log("Websocket keep alive check failed, no reply for %s ms", t - this.lastPong);
                    this.onDisconnected();
                    return;
                }
                if (this.socket !== undefined && this.socket.readyState === this.socket.OPEN) {
                    this.socket.send(this.makeReqNoSig(this.gameAddr, 'ping', {}));
                }
            }, 3000);
            if (this.streamResolve !== undefined) {
                let r = this.streamResolve;
                this.streamResolve = undefined;
                r(frame);
            }
            else {
                this.streamMessageQueue.push(frame);
            }
        };
        this.socket.onclose = () => {
            console.log('Websocket closed');
            this.closed = true;
            this.onDisconnected();
        };
        this.socket.onerror = (e) => {
            console.error(e);
            this.onDisconnected();
        };
        // Call JSONRPC subscribe_event
        const req = this.makeReqNoSig(this.gameAddr, 'subscribe_event', params);
        await this.requestWs(req);
    }
    async attachGame(params) {
        const req = this.makeReqNoSig(this.gameAddr, 'attach_game', params);
        await this.requestXhr(req);
    }
    async submitEvent(params) {
        try {
            const req = await this.makeReq(this.gameAddr, 'submit_event', params);
            await this.requestXhr(req);
            return undefined;
        }
        catch (_) {
            return 'disconnected';
        }
    }
    async submitMessage(params) {
        try {
            const req = await this.makeReq(this.gameAddr, 'submit_message', params);
            await this.requestXhr(req);
            return undefined;
        }
        catch (_) {
            return 'disconnected';
        }
    }
    async exitGame(params) {
        const req = await this.makeReq(this.gameAddr, 'exit_game', {});
        await this.requestXhr(req);
        if (!params.keepConnection) {
            if (this.socket !== undefined) {
                this.closed = true;
                this.socket.close();
                this.socket = undefined;
            }
        }
    }
    async *subscribeEvents() {
        await this.waitSocketReady();
        this.streamMessagePromise = new Promise(r => (this.streamResolve = r));
        while (true) {
            if (this.streamMessageQueue.length > 0) {
                yield this.streamMessageQueue.shift();
            }
            else {
                yield this.streamMessagePromise;
                this.streamMessagePromise = new Promise(r => (this.streamResolve = r));
            }
        }
    }
    parseEventMessage(raw) {
        let resp = JSON.parse(raw);
        if (resp.result === 'pong') {
            this.lastPong = new Date().getTime();
            return undefined;
        }
        else if (resp.method === 's_event') {
            if (resp.params.error === undefined) {
                let result = resp.params.result;
                let data = base64ToUint8Array(result);
                let frame = deserialize(BroadcastFrame, data);
                return frame;
            }
            else {
                return 'disconnected';
            }
        }
        else {
            return undefined;
        }
    }
    static initialize(gameAddr, playerAddr, endpoint, encryptor) {
        return new Connection(gameAddr, playerAddr, endpoint, encryptor);
    }
    async makeReq(gameAddr, method, params) {
        const paramsBytes = serialize(params);
        const sig = await this.encryptor.sign(paramsBytes, this.playerAddr);
        const sigBytes = serialize(sig);
        return JSON.stringify({
            jsonrpc: '2.0',
            method,
            id: crypto.randomUUID(),
            params: [gameAddr, arrayBufferToBase64(paramsBytes), arrayBufferToBase64(sigBytes)],
        });
    }
    makeReqNoSig(gameAddr, method, params) {
        const paramsBytes = serialize(params);
        return JSON.stringify({
            jsonrpc: '2.0',
            method,
            id: crypto.randomUUID(),
            params: [gameAddr, arrayBufferToBase64(paramsBytes)],
        });
    }
    async requestWs(req) {
        try {
            await this.waitSocketReady();
            if (this.socket !== undefined) {
                this.socket.send(req);
            }
        }
        catch (err) {
            console.error('Failed to connect to current transactor: ' + this.endpoint);
            throw err;
        }
    }
    async requestXhr(req) {
        try {
            const resp = await fetch(this.endpoint.replace(/^ws/, 'http'), {
                method: 'POST',
                body: req,
                headers: {
                    'Content-Type': 'application/json',
                },
            });
            if (resp.ok) {
                return resp.json();
            }
            else {
                throw Error('Transactor request failed:' + resp.json());
            }
        }
        catch (err) {
            console.error('Failed to connect to current transactor: ' + this.endpoint);
            throw err;
        }
    }
    waitSocketReady() {
        return new Promise((resolve, reject) => {
            let maxAttempts = 10;
            let intervalTime = 200;
            let currAttempt = 0;
            const interval = setInterval(() => {
                if (currAttempt > maxAttempts) {
                    clearInterval(interval);
                    reject();
                }
                else if (this.socket !== undefined && this.socket.readyState === this.socket.OPEN) {
                    clearInterval(interval);
                    resolve(undefined);
                }
                currAttempt++;
            }, intervalTime);
        });
    }
}
//# sourceMappingURL=connection.js.map